from invoke import Collection, task


CONFIG = {
    "git": {
        "token": None,
    }
}


@task
def config(ctx, token=None):
    """Setup git credentials with a Github token."""
    token = token or ctx.git.token
    assert token is not None, "Missing Github tokem."
    ctx.run("git config --global --unset url.ssh://git@github.com.insteadof")
    ctx.run(f"echo https://{token}:@github.com > ~/.git-credentials")
    ctx.run("git config --global credential.helper store")


ns = Collection("git")
ns.configure(CONFIG)
ns.add_task(config)



from invoke import task

from . import utils


# Local development workflow


@task(help={"force": "Whether to destroy the existing file first"})
def dotenv(ctx, template="./dotenv.tpl", target="./.env", force=False):
    """Create local dotenv file."""
    utils.check_path(template)
    try:
        utils.check_path(target)
        if force:
            raise utils.PathNotFound
    except utils.PathNotFound:
        ctx.run(f"cp {template} {target}")



import enum
import os
from typing import Optional

from invoke import Collection, Context, task

from . import utils


CONFIG = {
    "docker": {
        # Sensitive
        "repo": None,
        "token": None,
        "arg": None,
        # Non-sensitive
        "user": "AWS",
        "name": "webserver",
        "file": "Dockerfile",
        "context": ".",
        "tag": "test",
    }
}


class UserType(str, enum.Enum):
    AWS = "AWS"


# Docker deployment workflow:


@task
def login(ctx, repo=None, user=None, token=None):
    """Login to Docker remote registry (AWS ECR or Dockerhub)."""
    user = user or ctx.docker.user
    if user == UserType.AWS:
        _aws_login(ctx, repo)
    else:
        _dockerhub_login(ctx, user, token)


def _aws_login(ctx: Context, repo: Optional[str]) -> None:
    repo = repo or ctx.docker.repo
    assert repo is not None, "Missing remote AWS ECR URL."
    cmd = "aws ecr get-login-password | "
    cmd += f"docker login --username AWS --password-stdin {repo}"
    ctx.run(cmd)


def _dockerhub_login(ctx: Context, user: str, token: Optional[str]) -> None:
    token = token or ctx.docker.token
    assert token is not None, "Missing remote Dockerhub token."
    ctx.run(f"docker login --username {user} --password {token}")


@task
def build(ctx, name=None, file=None, context=None, arg=None):
    """Build Docker image locally."""
    name = name or ctx.docker.name
    context = context or ctx.docker.context
    file = file or f"{context}/{ctx.docker.file}"
    arg = arg or ctx.docker.arg
    utils.check_path(file)
    utils.check_path(context)
    cmd = f"docker build --pull --file {file} --tag {name} "
    if arg is not None:
        assert arg in os.environ, f"Missing environment variable {arg}."
        cmd += f"--build-arg {arg}={os.environ[arg]} "
    cmd += f"{context}"
    ctx.run(cmd)


@task(help={"dry-run": "Whether to tag the Docker image only"})
def push(ctx, repo=None, name=None, tag=None, dry_run=False):
    """Push Docker image to a remote registry."""
    repo = repo or ctx.docker.repo
    name = name or ctx.docker.name
    tag = tag or ctx.docker.tag
    for t in [tag, "latest"]:
        target_name = f"{repo}/{name}:{t}"
        ctx.run(f"docker tag {name} {target_name}")
        if not dry_run:
            ctx.run(f"docker push {target_name}")


ns = Collection("docker")
ns.configure(CONFIG)
ns.add_task(login)
ns.add_task(build)
ns.add_task(push)



from typing import Optional

from invoke import Collection, Context, task


CONFIG = {
    "terraform": {
        "organisation": None,
        "workspace": None,
        "token": None,
    }
}


# Terraform deployment workflow


@task
def update(ctx, variable="", value="", organisation=None, workspace=None, token=None):
    """Update variable in Terraform cloud."""
    cmd = f"noostf update --variable {variable} --value '{value}'"
    ctx.run(cmd + _append_credentials(ctx, organisation, workspace, token), pty=True)


@task
def run(ctx, message="", organisation=None, workspace=None, token=None):
    """Run a plan in Terraform cloud."""
    cmd = f"noostf run --message '{message}'"
    ctx.run(cmd + _append_credentials(ctx, organisation, workspace, token), pty=True)


def _append_credentials(
    ctx: Context, organisation: Optional[str], workspace: Optional[str], token: Optional[str]
) -> str:
    cmd = ""
    for arg in ctx.terraform:
        # Check credentials
        secret = locals()[arg] or ctx.terraform[arg]
        assert secret is not None, f"Missing Terraform Cloud {arg}."
        # Return credentials args
        cmd += f" --{arg} {secret}"
    return cmd


ns = Collection("terraform")
ns.configure(CONFIG)
ns.add_task(update)
ns.add_task(run)



import enum
from typing import Optional

from invoke import Collection, Context, task

from . import utils


CONFIG = {
    "python": {
        "install": "pipenv",
        "source": "./src",
        "tests": "./src/tests",
        "user": None,
        "token": None,
    }
}


class InstallType(str, enum.Enum):
    poetry = "poetry"
    pipenv = "pipenv"

    @classmethod
    def get(cls, ctx: Context, value: Optional[str]) -> str:
        install = value or ctx.python.install
        assert install in cls.__members__, f"Unknown Python installation {install}."
        return install


class GroupType(str, enum.Enum):
    unit = "unit"
    integration = "integration"
    functional = "functional"


# Python deployment workflow


@task
def clean(ctx):
    """Clean project from temp files / dirs."""
    ctx.run("rm -rf build dist")
    ctx.run("find src -type d -name __pycache__ | xargs rm -rf")


@task
def format(ctx, source=None, install=None):
    """Auto-format source code."""
    source = source or ctx.python.source
    utils.check_path(source)
    cmd = _activate_shell(ctx, install)
    ctx.run(cmd + f"black {source}", pty=True)
    ctx.run(cmd + f"isort {source}", pty=True)


@task
def lint(ctx, source=None, install=None):
    """Run python linters."""
    source = source or ctx.python.source
    utils.check_path(source)
    cmd = _activate_shell(ctx, install)
    ctx.run(cmd + f"black --check {source}", pty=True)
    ctx.run(cmd + f"isort --check-only {source}", pty=True)
    ctx.run(cmd + f"pydocstyle {source}", pty=True)
    ctx.run(cmd + f"flake8 {source}", pty=True)
    ctx.run(cmd + f"mypy {source}", pty=True)


@task
def test(ctx, tests=None, group="", install=None):
    """Run pytest with optional grouped tests."""
    tests = tests or ctx.python.tests
    if group != "":
        assert group in GroupType.__members__, f"Unknown py.test group {group}."
        tests += "/" + group
    utils.check_path(tests)
    cmd = _activate_shell(ctx, install)
    ctx.run(cmd + f"pytest {tests}", pty=True)


@task
def coverage(ctx, config="setup.cfg", report="term", tests=None, install=None):
    """Run coverage test report."""
    tests = tests or ctx.python.tests
    utils.check_path(tests)
    cmd = _activate_shell(ctx, install)
    ctx.run(cmd + f"pytest --cov --cov-config={config} --cov-report={report} {tests}", pty=True)


@task
def package(ctx, install=None):
    """Build project wheel distribution."""
    install_type = InstallType.get(ctx, install)
    if install_type == InstallType.poetry:
        ctx.run("poetry build", pty=True)
    if install_type == InstallType.pipenv:
        ctx.run("pipenv run python setup.py sdist", pty=True)


@task
def release(ctx, user=None, token=None, install=None):
    """Publish wheel distribution to PyPi."""
    user = user or ctx.python.user
    token = token or ctx.python.token
    assert user is not None, "Missing remote PyPi registry user."
    assert token is not None, "Missing remote PyPi registry token."
    install_type = InstallType.get(ctx, install)
    if install_type == InstallType.poetry:
        ctx.run(f"poetry publish --build -u {user} -p {token}", pty=True)
    if install_type == InstallType.pipenv:
        raise NotImplementedError


def _activate_shell(ctx: Context, install: str) -> str:
    install_type = InstallType.get(ctx, install)
    return f"{install_type} run "


ns = Collection("python")
ns.configure(CONFIG)
ns.add_task(clean)
ns.add_task(format)
ns.add_task(lint)
ns.add_task(test)
ns.add_task(coverage)
ns.add_task(package)
ns.add_task(release)



# Using `invoke` as a library
# http://docs.pyinvoke.org/en/stable/concepts/library.html

from invoke import Collection, Config, Program

from . import docker, git, helm, local, python, terraform


__version__ = "0.0.7"


class BaseConfig(Config):
    prefix = "noosinv"


ns = Collection()
ns.add_collection(docker.ns)
ns.add_collection(git.ns)
ns.add_collection(helm.ns)
ns.add_collection(local)
ns.add_collection(python.ns)
ns.add_collection(terraform.ns)


main = Program(namespace=ns, config_class=BaseConfig, version=__version__)



import pathlib


class PathNotFound(Exception):
    pass


def check_path(path: str) -> None:
    """Check whether a path exists on a file system."""
    if not pathlib.Path(path).exists():
        raise PathNotFound(f"Incorrect file system path: {path}")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['noos_inv']

package_data = \
{'': ['*']}

install_requires = \
['invoke']

entry_points = \
{'console_scripts': ['noosinv = noos_inv:main.run']}

setup_kwargs = {
    'name': 'noos-inv',
    'version': '0.0.7',
    'description': 'Shared workflows across CI/CD pipelines',
    'long_description': '[![CircleCI](https://circleci.com/gh/noosenergy/noos-invoke.svg?style=svg&circle-token=68d1a71e4f53ab1a1f33110e9a8c24bd3300a8ba)](https://circleci.com/gh/noosenergy/noos-invoke)\n\n# Noos Invoke\n\nSoftware development kit for sharing workflows across CI/CD pipelines.\n\nSuch a project aims to enforce parity and reproducability between local development and CI/CD workflows in remote containers (e.g. executable versions, command line calls, environment variables...) - developped with `inv[oke]`(https://github.com/pyinvoke/invoke).\n\n## Installation\n\nInstall the package from the [PyPi repository](https://pypi.org/project/noos-inv/):\n\n    $ pip install noos-inv\n\nTo enable shell completion, execute the following command (e.g. `zsh`),\n\n    $ noosinv --print-completion-script=zsh\n\nAnd copy/paste its `stdout` into your shell config.\n\n```bash\n# NOOSINV completion script\n\n_complete_noosinv() {\n    collection_arg=\'\'\n    if [[ "${words}" =~ "(-c|--collection) [^ ]+" ]]; then\n        collection_arg=$MATCH\n    fi\n    reply=( $(noosinv ${=collection_arg} --complete -- ${words}) )\n}\n\ncompctl -K _complete_noosinv + -f noosinv\n```\n\nFinally, still in your shell config, enable automatic sub shell loading:\n\n```bash\n# ENV variable sub shell loading with command "source .env"\n\nset -a\n```\n\n## Usage as a command line tool\n\nThe `noos-inv` package installs a CLI binary, for managing common CI/CD tasks.\n\nFrom the terminal,\n\n```\n$ noosinv\n\nUsage: noosinv [--core-opts] <subcommand> [--subcommand-opts] ...\n\nSubcommands:\n\n  docker.build       Build Docker image locally.\n  docker.login       Login to Docker remote registry (AWS ECR or Dockerhub).\n  docker.push        Push Docker image to a remote registry.\n  git.config         Setup git credentials with a Github token.\n  helm.install       Provision local Helm client (Chart Museum Plugin).\n  helm.lint          Check compliance of Helm charts / values.\n  helm.login         Login to Helm remote registry (Chart Museum).\n  helm.push          Push Helm chart to a remote registry.\n  helm.test          Test local deployment in Minikube.\n  local.dotenv       Create local dotenv file.\n  python.clean       Clean project from temp files / dirs.\n  python.coverage    Run coverage test report.\n  python.format      Auto-format source code.\n  python.lint        Run python linters.\n  python.package     Build project wheel distribution.\n  python.release     Publish wheel distribution to PyPi.\n  python.test        Run pytest with optional grouped tests.\n  terraform.run      Run a plan in Terraform cloud.\n  terraform.update   Update variable in Terraform cloud.\n```\n\nSource your environnement variables first for a seamless experience.\n(use command `local.dotenv` to create it from the provided template)\n\n    $ source .env\n\n## Development\n\nOn Mac OSX, make sure [poetry](https://python-poetry.org/) has been installed and pre-configured,\n\n    $ brew install poetry\n\nThis project is shipped with a Makefile, which is ready to do basic common tasks.\n\n```\n$ make\n\nhelp                           Display this auto-generated help message\nupdate                         Lock and install build dependencies\nclean                          Clean project from temp files / dirs\nformat                         Run auto-formatting linters\ninstall                        Install build dependencies from lock file\nlint                           Run python linters\ntest                           Run pytest with all tests\npackage                        Build project wheel distribution\nrelease                        Publish wheel distribution to PyPi\n```\n',
    'author': 'Noos Energy',
    'author_email': 'contact@noos.energy',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/noosenergy/noos-invoke',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from invoke import Collection, task

from . import utils


CONFIG = {
    "helm": {
        "repo": "local-repo",
        "url": None,
        "user": None,
        "token": None,
        "plugins": ["https://github.com/chartmuseum/helm-push.git"],
        "chart": "./helm/chart",
        "values": "./local/helm-values.yaml",
    }
}


# Helm deployment workflow:


@task
def login(ctx, repo=None, url=None, user=None, token=None):
    """Login to Helm remote registry (Chart Museum)."""
    repo = repo or ctx.helm.repo
    url = url or ctx.helm.url
    user = user or ctx.helm.user
    token = token or ctx.helm.token
    assert url is not None, "Missing remote Helm registry url."
    assert user is not None, "Missing remote Helm registry user."
    assert token is not None, "Missing remote Helm registry token."
    ctx.run(f"helm repo add {repo} {url} --username {user} --password {token}")


@task(iterable=["plugins"])
def install(ctx, plugins=None):
    """Provision local Helm client (Chart Museum Plugin)."""
    plugins = plugins or ctx.helm.plugins
    for plugin in plugins:
        ctx.run(f"helm plugin install {plugin}")


@task
def lint(ctx, chart=None):
    """Check compliance of Helm charts / values."""
    chart = chart or ctx.helm.chart
    utils.check_path(chart)
    ctx.run(f"helm lint {chart}")


@task(help={"dry-run": "Whether to render the Helm manifest first"})
def test(
    ctx,
    chart=None,
    values=None,
    release="test",
    namespace="default",
    context="minikube",
    dry_run=False,
):
    """Test local deployment in Minikube."""
    chart = chart or ctx.helm.chart
    values = values or ctx.helm.values
    utils.check_path(chart)
    utils.check_path(values)
    cmd = f"helm install {release} {chart} --values {values} "
    cmd += f"--create-namespace --namespace {namespace} --kube-context {context}"
    if dry_run:
        cmd += " --dry-run --debug"
    ctx.run(cmd)


@task
def push(ctx, chart=None, repo=None):
    """Push Helm chart to a remote registry."""
    chart = chart or ctx.helm.chart
    repo = repo or ctx.helm.repo
    utils.check_path(chart)
    ctx.run(f"helm push {chart} {repo} --dependency-update")


ns = Collection("helm")
ns.configure(CONFIG)
ns.add_task(login)
ns.add_task(install)
ns.add_task(lint)
ns.add_task(test)
ns.add_task(push)
